from django.shortcuts import render
from clhrApp.views import *
# Create your views here.

class BranchEmployeesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_BranchEmployees'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        # if not branchId : return redirect('clauthApp:home')
        departments =Department.objects.all()
        branches = Branch.objects.filter(id=branchId)
        # departments =Department.objects.filter(is_clinic=False)
        all_emps =Employee.objects.filter(branch_id=branchId).order_by("-is_active")
        all_emps_srl=EmployeeSerializer(all_emps,many=True)
        return render(request,"Branch/brhrApp/branch_employees.html",{"all_emps":all_emps_srl.data,"departments":departments,"branches":branches,
        'activePage':'BranchEmployees'})
    

class EmployeeApi(APIView):
    def get_object(self,pk):
        try:
            return Employee.objects.get(pk=pk)
        except Employee.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_data_BranchEmployee'), login_url='/main/login/'))        
    def get(self,request,pk): 
        empObj=self.get_object(pk)
        if empObj :
            context = {"emp":empObj,'activePage':'Employees'}
            all_earnings = Earning.objects.all()
            all_deductions = Deduction.objects.all()
            context.update({"all_earnings":all_earnings,"all_deductions": all_deductions})
            empSalaries = EmpSalary.objects.filter(emp_id=pk).order_by('-to_day')
            not_paid_salary = empSalaries.filter(pay_day__isnull=True).count()
            max_pay_day = empSalaries.aggregate(Max('pay_day'))['pay_day__max']
            max_pay_dayObjs = empSalaries.filter(pay_day = max_pay_day)
            if max_pay_dayObjs : 
                last_to_day = max_pay_dayObjs[0].to_day
                day_after_last_to_day = last_to_day+timedelta(days=1)
            else : day_after_last_to_day = None
            context.update({
            'activePage':'BranchEmployees',
            "all_emp_salaries":empSalaries ,
             'day_after_last_to_day':day_after_last_to_day,
             'emp_payday_max':empSalaries.aggregate(Max('pay_day'))['pay_day__max'],
             "not_paid_salary":not_paid_salary})
            empEarnings = EmpEarning.objects.filter(emp_id=pk ,is_done=False).order_by('-created_at')
            context.update({"all_emp_earnings":empEarnings , 'emp_earnings_sum':empEarnings.aggregate(Sum('amount'))['amount__sum']})
            empDeduction = EmpDeduction.objects.filter(emp_id=pk, is_done=False).order_by('-created_at')
            context.update({"all_emp_deductions":empDeduction , 'emp_deductions_sum':empDeduction.aggregate(Sum('amount'))['amount__sum']})
            # patientVisitObjs = PatientVisit.objects.filter(patient=patientObj,visit_status__in=["1I","4F"]).order_by('-visit_day')
            # patientVisitObjs_serialized = PatientVisitMiniSerializer(patientVisitObjs,many=True)
            # context.update({'patientVisitObjs':patientVisitObjs_serialized.data})
            # medical_services =MedicalService.objects.all().values('id', 'name','price')
            # context.update({"medical_services":medical_services,})           
            # blood_type_choices = ChoiceField(choices=BLOOD_TYPE_CHOICES)
            # context.update({"blood_types":blood_type_choices.choices,})
            # context.update({"pageTitle":patientObj.fullname,})
            return render(request,"Branch/brhrApp/employee.html",context)
        else : return redirect('/main/login/')

class DrLevelsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_DrLevels'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        dr_levels = DrLevel.objects.filter(branch_id=branchId)
        return render(request,"Branch/brhrApp/dr_levels.html",{"dr_levels":dr_levels,'activePage':'DrLevels'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_DrLevel'), login_url='/main/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        data = request.data
        data._mutable = True
        #data.update({"created_by":request.user.id,"branch":branchId})
        data.update({"branch":branchId})
        serializedObj= DrLevelSerializer(data=data)
        default_level_brnach = DrLevel.objects.filter(branch_id=branchId,is_default=True)
        if data["is_default"]== "True" and default_level_brnach : 
            DrLevel.objects.filter(branch_id=branchId).update(is_default=False)
            data["is_default"] = True
        elif  data["is_default"]== "False" and not default_level_brnach: 
            data["is_default"] = True
        #list_error=""
        import pdb
        #pdb.set_trace()
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class DrLevelApi(APIView):
    def get_object(self,pk):
        try:
            return DrLevel.objects.get(pk=pk)
        except DrLevel.DoesNotExist:
            return None
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_DrLevels'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        medical_serviceObj=self.get_object(pk)
        if medical_serviceObj :
            serializeobj= DrLevelSerializer(medical_serviceObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_DrLevel'), login_url='/auth/login/'))
    def put(self,request,pk):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        obj=self.get_object(pk)
        data = request.data
        data._mutable = True
        #request.data.update({"created_by":request.user.id})
        default_level_brnach = DrLevel.objects.filter(branch_id=branchId,is_default=True)
        if data["is_default"]== "True" and default_level_brnach : 
            DrLevel.objects.filter(branch_id=branchId).update(is_default=False)
            data["is_default"] = True
        elif  data["is_default"]== "False" and not default_level_brnach:data["is_default"] = True
        elif  data["is_default"]== "False" and default_level_brnach:data["is_default"] = True
        serializedObj= DrLevelSerializer(obj,data=data,partial=True)
        #serializedObj= DrLevelSerializer(obj,data=request.data,partial=True)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

            
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_DrLevel'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

##### emp_earnings  استحقاق لموظف او مكافأة أو مرتب ########

class EmpEarningsApi(APIView):
    #next step مش هعرض في الصفحة علي طول هعمل loading ومتظهرش إلا لما يضغط عليها 
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_Exercise'), login_url='/main/login/'))
    def get(self,request,emp_id):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        emp_ern =EmpEarning.objects.all() #,is_done=False
        emp_ern_srl=EmpEarningSerializer(emp_ern,many=True)
        return render(request,"clhrApp/Managment/exercises.html",{"emp_ern":emp_ern_srl.data,
        'activePage':'EmpEarnings'})

    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_EmpEarning'), login_url='/main/login/'))    
    def post(self,request,emp_id):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        request.data._mutable = True
        request.data.update({"emp":emp_id})
        request.data.update({"earning_note":None if (not request.data["earning_note"]) else request.data["earning_note"] })       
        request.data.update({"is_done":False})       
        request.data.update({"created_by":request.user.id})
        serializedObj= EmpEarningSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EmpEarningApi(APIView):
    def get_object(self,pk):
        try:
            return EmpEarning.objects.get(pk=pk)
        except EmpEarning.DoesNotExist:
            return None    
    
    # @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_EmpEarning'), login_url='/main/login/'))        
    # def get(self,request,pk): 
    #     exrszObj=self.get_object(pk)
    #     if exrszObj :
    #         serializeobj= EmpEarningSerializer(exrszObj)
    #         return Response(serializeobj.data,status=status.HTTP_200_OK)
    #     else : return Response(status=status.HTTP_404_NOT_FOUND)

    # @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.edit_EmpEarning'), login_url='/main/login/'))
    # def put(self,request,pk):
    #     obj=self.get_object(pk)
    #     serializedObj= EmpEarningSerializer(obj,data=request.data)
    #     if serializedObj.is_valid():
    #         serializedObj.save()
    #         return Response(status=status.HTTP_200_OK)
    #     else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_EmpEarning'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response("Object Deleted By another User !! Reload Your page ..", status=status.HTTP_404_NOT_FOUND)



#https://stackoverflow.com/questions/29858701/django-query-isnull-true-or-none
# حساب القيمة المبدئية للراتب من بيانات الحضور والإنصراف والعلاوات والخصومات
class EmpSalaryDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_MedServiceReportDoctor'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        medical_services = MedicalService.objects.all().values("id","name")
        departments=  Department.objects.all().order_by("d_order")
        doctors =Doctor.objects.filter(is_active=True).values("id","name","department")
        return render(request,"mainApp/financePages/med_service/doctor.html",{
        "medical_services":medical_services,"doctors":doctors,
        "departments":departments,'activePage':'MedServiceReportDoctor'})
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_MedServiceReportDoctor'), login_url='/main/login/'))    
    def post(self,request,emp_id):
            branchId = getBranchId(request)
            if not branchId : return redirect('clauthApp:home')
            data=request.data
            startDate =  datetime.strptime(data["startDate"], '%Y-%m-%d') 
            endDate =datetime.strptime(data["endDate"], '%Y-%m-%d') .replace(hour=23,minute=59,second=59,microsecond=59)
            attendace_data = EmpAttendance.objects.filter(emp_id =emp_id,check_out_by__isnull=False,
                day__gte = startDate, day__lte=endDate,is_done=False,salary__isnull=True).order_by("day")
            total_empEarning = EmpEarning.objects.filter(emp_id =emp_id,created_at__gte = startDate, created_at__lte=endDate,is_done=False,salary__isnull=True).aggregate(Sum('amount'))['amount__sum']
            total_empDeduction = EmpDeduction.objects.filter(emp_id =emp_id,created_at__gte = startDate, created_at__lte=endDate,is_done=False,salary__isnull=True).aggregate(Sum('amount'))['amount__sum']

            #else : 
                #visits = PatientVisit.objects.filter(created_at__gte = startDateTime , created_at__lte=endDateTime,doctor_id=doctor,medicalservice_id=medicalservice,visit_status="4F").order_by("created_at")
            attendace_serializer= EmpAttendanceSerializer(attendace_data,many=True)
            return Response({"attendace_data":attendace_serializer.data,
            "total_empEarning":total_empEarning,"total_empDeduction":total_empDeduction},
            status=status.HTTP_200_OK)

class EmpSalaryApi(APIView):
    def get_object(self,pk):
        try:
            return EmpSalary.objects.get(pk=pk)
        except EmpSalary.DoesNotExist:
            return None   
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_EmpEarning'), login_url='/main/login/'))    
    def post(self,request,emp_id):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        request.data._mutable = True
        request.data.update({"emp":emp_id})    
        request.data.update({"created_by":request.user.id})
        data=request.data
        #startDate = data["from_day"]
        #endDate = data["to_day"]
        att__ids = data.getlist('att__ids[]')
        ee__ids = data.getlist('ee__ids[]')
        ed__ids = data.getlist('ed__ids[]')
        serializedObj= EmpSalarySerializer(data=data)
        if serializedObj.is_valid():
            serializedObj.save()
            salary_id = serializedObj.data["id"]
            EmpAttendance.objects.filter(id__in=att__ids).update(salary_id=salary_id,is_done=True)
            EmpEarning.objects.filter(id__in=ee__ids).update(salary_id=salary_id,is_done=True)
            EmpDeduction.objects.filter(id__in=ed__ids).update(salary_id=salary_id,is_done=True)
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
    
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.edit_Job'), login_url='/main/login/'))
    def put(self,request,emp_id):
        request.data._mutable = True
        request.data.update({"pay_day":timezone.now().date()})    
        request.data.update({"delivered_by":request.user.id})
        data=request.data
        salaryId = data["emp_salary_id"]
        obj=self.get_object(salaryId)
        serializedObj= EmpSalarySerializer(obj,data=data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_EmpDeduction'), login_url='/main/login/'))
    def delete(self,request,emp_id):
        data=request.data
        salaryId = data["emp_salary_id"]
        emp_salary=self.get_object(salaryId)
        EmpAttendance.objects.filter(salary_id=salaryId).update(salary=None,is_done=False)
        EmpEarning.objects.filter(salary_id=salaryId).update(salary=None,is_done=False)
        EmpDeduction.objects.filter(salary_id=salaryId).update(salary=None,is_done=False)
        if emp_salary :
            try:
                emp_salary.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

##### emp_deductions  خصم لموظف########
class EmpDeductionsApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_EmpDeduction'), login_url='/main/login/'))    
    def post(self,request,emp_id):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        request.data._mutable = True
        request.data.update({"emp":emp_id})
        request.data.update({"deduction_note":None if (not request.data["deduction_note"]) else request.data["deduction_note"] })       
        request.data.update({"is_done":False})       
        request.data.update({"created_by":request.user.id})
        serializedObj= EmpDeductionSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class EmpDeductionApi(APIView):
    def get_object(self,pk):
        try:
            return EmpDeduction.objects.get(pk=pk)
        except EmpDeduction.DoesNotExist:
            return None   
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_EmpDeduction'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

# class EmpDeductionsApi(APIView):
#     @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_Exercise'), login_url='/main/login/'))
#     def get(self,request,emp_pk):
#                 branchId = getBranchId(request)
#        if not branchId : return redirect('clauthApp:home')
#         emp_ds =EmpDeduction.objects.filter(emp_id=emp_pk,is_done=False)
#         emp_ds_srl=EmpDeductionSerializer(emp_ds,many=True)
#         return Response(emp_ds_srl.data,status=status.HTTP_200_OK)
#     @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_Exercise'), login_url='/main/login/'))    
#     def post(self,request,emp_pk):
#                 branchId = getBranchId(request)
#        if not branchId : return redirect('clauthApp:home')
#         request.data._mutable = True
#         request.data.update({"created_by":request.user.id})
#         serializedObj= EmpDeductionSerializer(data=request.data)
#         if serializedObj.is_valid():
#             serializedObj.save()
#             return Response(serializedObj.data,status=status.HTTP_201_CREATED)
#         else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

# class EmpDeductionApi(APIView):
#     def get_object(self,pk):
#         try:
#             return EmpDeduction.objects.get(pk=pk)
#         except EmpDeduction.DoesNotExist:
#             return None    
    
#     @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_EmpDeduction'), login_url='/main/login/'))        
#     def get(self,request,pk): 
#         exrszObj=self.get_object(pk)
#         if exrszObj :
#             serializeobj= EmpDeductionSerializer(exrszObj)
#             return Response(serializeobj.data,status=status.HTTP_200_OK)
#         else : return Response(status=status.HTTP_404_NOT_FOUND)

#     @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.edit_EmpDeduction'), login_url='/main/login/'))
#     def put(self,request,pk):
#         obj=self.get_object(pk)
#         serializedObj= EmpDeductionSerializer(obj,data=request.data)
#         if serializedObj.is_valid():
#             serializedObj.save()
#             return Response(status=status.HTTP_200_OK)
#         else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

#     @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_EmpDeduction'), login_url='/main/login/'))
#     def delete(self,request,pk):
#         obj=self.get_object(pk)
#         if obj :
#             try:
#                 obj.delete()
#                 return Response(status=status.HTTP_204_NO_CONTENT)
#             except Exception as err_msg: 
#                     return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
#         else : return Response(status=status.HTTP_404_NOT_FOUND)